<?php

declare(strict_types=1);

namespace Drupal\verify_email\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\verify_email\VerifyEmailInterface;

/**
 * Defines the verify email form entity type.
 *
 * @ConfigEntityType(
 *   id = "verify_email",
 *   label = @Translation("Verify Email Form"),
 *   label_collection = @Translation("Verify Email Forms"),
 *   label_singular = @Translation("Verify Email form"),
 *   label_plural = @Translation("Verify Email forms"),
 *   label_count = @PluralTranslation(
 *     singular = "@count verify email form",
 *     plural = "@count verify email forms",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\verify_email\VerifyEmailListBuilder",
 *     "form" = {
 *       "add" = "Drupal\verify_email\Form\VerifyEmailAddEditForm",
 *       "edit" = "Drupal\verify_email\Form\VerifyEmailAddEditForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *   },
 *   config_prefix = "verify_email",
 *   admin_permission = "administer verify email forms",
 *   links = {
 *     "collection" = "/admin/people/verify-email",
 *     "edit-form" = "/admin/people/verify-email/{verify_email}",
 *     "delete-form" = "/admin/people/verify-email/{verify_email}/delete",
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "path",
 *     "intro",
 *     "email_subject",
 *     "email_body",
 *     "thank_you_body",
 *     "redirect_path",
 *   }
 * )
 */
final class VerifyEmail extends ConfigEntityBase implements VerifyEmailInterface {

  /**
   * The example ID.
   */
  protected string $id;

  /**
   * The example label.
   */
  protected string $label;

  /**
   * The path this search page will appear upon.
   *
   * @var string
   */
  protected $path;

  /**
   * {@inheritdoc}
   */
  public function getPath(): string {
    return $this->path ?? '';
  }

  /**
   * The form intro text.
   */
  protected string $intro;

  /**
   * {@inheritdoc}
   */
  public function getIntro(): string {
    return $this->intro ?? '';
  }

  /**
   * The email subject.
   */
  protected string $email_subject;

  /**
   * {@inheritdoc}
   */
  public function getEmailSubject(): string {
    return $this->email_subject ?? '';
  }

  /**
   * The email message body.
   */
  protected string $email_body;

  /**
   * {@inheritdoc}
   */
  public function getEmailBody(): string {
    return $this->email_body ?? '';
  }

  /**
   * Custom thank you message.
   */
  protected string $thank_you_body;

  /**
   * {@inheritdoc}
   */
  public function getThankYouBody(): string {
    $default_value = (string) t('Please check your email and click on the included link to verify your email address.');
    return !empty($this->thank_you_body) ? $this->thank_you_body : $default_value;
  }

  /**
   * The redirect path.
   */
  protected string $redirect_path;

  /**
   * {@inheritdoc}
   */
  public function getRedirectPath(): string {
    return $this->redirect_path ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
    $route_builder = $this->routeBuilder();
    $route_builder->setRebuildNeeded();
    $route_builder->rebuild();
  }

  /**
   * Wraps the route builder.
   *
   * @return \Drupal\Core\Routing\RouteBuilderInterface
   *   An object for state storage.
   */
  protected function routeBuilder(): RouteBuilderInterface {
    return \Drupal::service('router.builder');
  }

}
