<?php

declare(strict_types=1);

namespace Drupal\verify_email\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\verify_email\Entity\VerifyEmail;
use Drupal\Core\Path\PathValidatorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Verification Add/Edit form.
 */
final class VerifyEmailAddEditForm extends EntityForm {

  /**
   * Constructs a new VerifyEmailAddEditForm.
   */
  public function __construct(
    protected PathValidatorInterface $pathValidator,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('path.validator'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {

    /** @var \Drupal\verify_email\VerifyEmailInterface */
    $entity = $this->entity;

    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => [VerifyEmail::class, 'load'],
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $entity->status(),
    ];

    $form['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#description' => $this->t('Enter the internal path where the verification form should be available. Include the leading slash (e.g., /verify-me), or use <front> for front page.'),
      '#default_value' => $entity->getPath(),
      '#maxlength' => '255',
      '#required' => TRUE,
    ];

    $form['intro'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Introduction text'),
      '#description' => $this->t('Text to display at the top of the verify email form.'),
      '#default_value' => $entity->getIntro(),
    ];

    $form['email_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email subject'),
      '#default_value' => $entity->getEmailSubject(),
      '#maxlength' => '255',
      '#required' => TRUE,
    ];

    $form['email_body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Email body'),
      '#description' => $this->t('Email message to be sent to user. Place token [verify-email:url] to insert the login link.'),
      '#default_value' => $entity->getEmailBody(),
      '#required' => TRUE,
    ];

    $form['thank_you_body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Thank You Message'),
      '#description' => $this->t('Text to display on thank you page.'),
      '#default_value' => $entity->getThankYouBody(),
    ];

    $form['redirect_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Redirect path'),
      '#description' => $this->t('The internal path the user will be redirected, including leading slash, or <front> to redirect to front page of site.'),
      '#default_value' => $entity->getRedirectPath(),
      '#maxlength' => '255',
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    $redirect_path = $form_state->getValue('redirect_path');
    if (!$this->pathValidator->isValid($redirect_path)) {
      $form_state->setErrorByName('redirect_path', $this->t('The redirect path %path is not valid.', ['%path' => $redirect_path]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $this->messenger()->addStatus(
      match($result) {
        \SAVED_NEW => $this->t('Created new example %label.', $message_args),
        \SAVED_UPDATED => $this->t('Updated example %label.', $message_args),
      }
    );
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

}
