<?php

declare(strict_types=1);

namespace Drupal\verify_email\Form;

use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\verify_email\Service\VerifierInterface;
use Drupal\verify_email\VerifyEmailInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides a Verify Email form.
 */
final class VerifyEmailForm extends FormBase {

  /**
   * Constructs a new VerifyEmailForm.
   */
  public function __construct(
    protected readonly AccountProxyInterface $currentUser,
    protected readonly EmailValidatorInterface $emailValidator,
    protected readonly VerifierInterface $verifier,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('current_user'),
      $container->get('email.validator'),
      $container->get('verify_email.verifier'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'verify_email_verify_email';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, VerifyEmailInterface|null $entity = NULL): RedirectResponse|array {

    $form_state->set('entity', $entity);
    if ($this->currentUser()->isAuthenticated()) {
      return new RedirectResponse($entity->getRedirectPath());
    }

    $form['intro'] = [
      '#markup' => $entity->getIntro(),
      '#prefix' => '<div class="verify-email-intro">',
      '#suffix' => '</div>',
    ];

    $form['email'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter your email address'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Verify'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Use the email validator service to validate the email address.
    $email = $form_state->getValue('email');
    if (!$this->emailValidator->isValid($email)) {
      $form_state->setErrorByName('email', $this->t('The email address %email is not valid.', ['%email' => $email]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $entity = $form_state->get('entity');
    $email = $form_state->getValue('email');
    // Send email containing confirmation link.
    $this->verifier->sendVerificationEmail($entity, $email);
    $this->messenger()->addStatus($this->t('The message has been sent.'));

    // Redirect to the 'thank you' page of the specific entity.
    $form_state->setRedirect('verify_email.' . $entity->id() . '.thanks');
  }

}
