<?php

declare(strict_types=1);

namespace Drupal\verify_email\Service;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\TempStore\SharedTempStoreFactory;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\user\Entity\User;
use Drupal\verify_email\LoginInfo;
use Drupal\verify_email\VerifyEmailInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class to verify email address by sending link.
 */
final class Verifier implements VerifierInterface {

  use MessengerTrait;

  // cspell:disable-next-line
  protected const CHARS = '0123456789abcdefghijklmnopqrtsuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';

  /**
   * Constructs a Verifier object.
   */
  public function __construct(
    private readonly VerifyEmailRepositoryInterface $repository,
    private readonly MailManagerInterface $mailManager,
    private readonly LanguageManagerInterface $languageManager,
    private readonly TimeInterface $time,
    private readonly SharedTempStoreFactory $tempStore,
    private readonly Token $token,
  ) {}

  /**
   * Create a string of random characters.
   *
   * @return string
   *   A random string.
   */
  protected function secret(): string {
    return implode('', array_map(fn($x) => self::CHARS[rand(0, 51)], array_fill(0, 10, '')));
  }

  /**
   * {@inheritdoc}
   */
  public function generateLink(VerifyEmailInterface $entity, string $emailAddress): Url {
    $store = $this->tempStore->get('verify_email');
    $key = rand(100_000_000, 999_999_999);
    $loginInfo = new LoginInfo(
      $entity->id(),
      $emailAddress,
      $this->secret(),
      $this->time->getCurrentTime() + 3600,
    );
    $store->set($key, $loginInfo);
    return Url::fromRoute('verify_email.verify_link', ['key' => $key, 'secret' => $loginInfo->secret], ['absolute' => TRUE]);
  }

  /**
   * {@inheritdoc}
   */
  public function sendVerificationEmail(VerifyEmailInterface $entity, string $emailAddress): void {
    $default_langcode = $this->languageManager->getDefaultLanguage()->getId();
    $url = $this->generateLink($entity, $emailAddress)->toString();

    $data = [
      'verify-email' => [
        'url' => $url,
      ],
    ];

    $subject = $this->token->replace($entity->getEmailSubject(), $data);
    $body = $this->token->replace($entity->getEmailBody(), $data);

    $mail_params = [
      'to' => $emailAddress,
      'subject' => $subject,
      'body' => trim($body),
    ];
    $this->mailManager->mail('verify_email', 'verify', $emailAddress, $default_langcode, $mail_params);
  }

  /**
   * If login key invalid, or verify email entity not found, go to front page.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirection to the front page.
   */
  protected function keyInvalid(): RedirectResponse {
    $this->messenger()->addWarning('Sorry, the verification could not be validated. Please try verifying again.');
    $url = Url::fromRoute('<front>');
    return new RedirectResponse($url->toString());
  }

  /**
   * {@inheritdoc}
   */
  public function verifyAndLogin(string $loginKey, string $user_secret): RedirectResponse {
    // If key is not numeric, vot valid.
    if (!is_numeric($loginKey)) {
      return $this->keyInvalid();
    }

    $keyVal = intval($loginKey);
    $store = $this->tempStore->get('verify_email');
    $loginInfo = $store->get($keyVal);

    // If not found in tempstore, not valid.
    if (!$loginInfo || !hash_equals($loginInfo->secret, $user_secret)) {
      return $this->keyInvalid();
    }

    // Fetch the VerifyEmail entity.
    /** @var \Drupal\verify_email\Entity\VerifyEmail */
    $entity = $this->repository->getVerifyEmailForm($loginInfo->id);

    // If expiry before current time, redirect back to verify form.
    if ($loginInfo->expiry < $this->time->getCurrentTime()) {
      $this->messenger()->addMessage('Login link has expired. Please enter your email address again to receive a new link.');
      return new RedirectResponse(Url::fromUri('internal:' . $entity->getPath())->toString());
    }

    // Fetch user from email address.
    $user = user_load_by_mail($loginInfo->email);
    // If user not found, create one.
    if (!$user) {
      $langcode = $this->languageManager->getCurrentLanguage()->getId();
      $user = User::create([
        'name' => $loginInfo->email,
        'mail' => $loginInfo->email,
        'langcode' => $langcode,
        'status' => 1,
      ]);
      $user->save();
    }
    // If user exists, log in.
    user_login_finalize($user);

    // User logged in. Redirect to destination.
    return new RedirectResponse(Url::fromUri('internal:' . $entity->getRedirectPath())->toString());
  }

}
