<?php

declare(strict_types=1);

namespace Drupal\verify_email;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a listing of verify email forms.
 */
final class VerifyEmailListBuilder extends ConfigEntityListBuilder implements FormInterface {

  /**
   * The theme name.
   *
   * @var string
   */
  private string $theme;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  private ?Request $request = NULL;

  /**
   * Constructs a new VerifyEmailListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   The theme manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
   *   The form builder.
   */
  public function __construct(
    EntityTypeInterface $entity_type,
    EntityStorageInterface $storage,
    private ThemeManagerInterface $themeManager,
    private FormBuilderInterface $formBuilder,
  ) {
    parent::__construct($entity_type, $storage);
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance($container, $entity_type): VerifyEmailListBuilder {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('theme.manager'),
      $container->get('form_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'verify_email_list_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param string|null $theme
   *   The theme to display the blocks for. If NULL, the current
   *   theme will be used.
   * @param \Symfony\Component\HttpFoundation\Request|null $request
   *   The current request. If NULL, the current request will be used.
   *
   * @return array
   *   The block list as a renderable array.
   */
  public function render($theme = NULL, ?Request $request = NULL): array {
    $this->theme = $theme ?? $this->themeManager->getActiveTheme()->getName();
    $this->request = $request;

    return $this->formBuilder->getForm($this);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Ensure entities are loaded.
    $entities = $this->load();

    $form['global_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Global Settings'),
      '#open' => TRUE,
    ];

    $form['global_settings']['placeholder'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Placeholder for future global options.'),
      '#prefix' => '<div class="global-options-placeholder">',
      '#suffix' => '</div>',
    ];

    $form['verify_email_forms'] = [
      '#type' => 'details',
      '#title' => $this->t('Verify Email Forms'),
      '#open' => TRUE,
    ];

    $form['verify_email_forms']['add_new'] = [
      '#type' => 'link',
      '#title' => $this->t('Add new Verify Email form'),
      '#url' => Url::fromRoute('entity.verify_email.add_form'),
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
    ];

    $form['verify_email_forms']['table'] = [
      '#type' => 'table',
      '#header' => $this->buildHeader(),
      '#rows' => [],
      '#empty' => $this->t('There are no Verify Email forms yet.'),
    ];

    foreach ($entities as $entity) {
      $form['verify_email_forms']['table']['#rows'][$entity->id()] = $this->buildRow($entity);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // No validation required for now.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // No submission logic required for now.
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    $header['label'] = ['data' => $this->t('Label')];
    $header['id'] = ['data' => $this->t('Machine name')];
    $header['url'] = ['data' => $this->t('URL')];
    $header['status'] = ['data' => $this->t('Status')];
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\verify_email\VerifyEmailInterface $entity */
    $row['label'] = $entity->label();
    $row['id'] = $entity->id();
    $row['url']['data']['#markup'] = $entity->getPath();
    if ($entity->status()) {
      $row['url']['data'] = [
        '#type' => 'link',
        '#title' => $row['url']['data'],
        '#url' => Url::fromRoute('verify_email.' . $entity->id()),
      ];
    }
    $row['status'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');
    return $row + parent::buildRow($entity);
  }

}
