<?php

declare(strict_types=1);

namespace Drupal\Tests\verify_email\Functional;

use Drupal\Core\Test\AssertMailTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\verify_email\LoginInfo;
use Drupal\verify_email\Entity\VerifyEmail;
use PHPUnit\Framework\Attributes\Group;

/**
 * Test description.
 */
#[Group('verify_email')]
final class VerifyEmailRoutingTest extends BrowserTestBase {

  use AssertMailTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'verify_email'];

  /**
   * {@inheritdoc}
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Set up the test here.
    $this->adminUser = $this->drupalCreateUser([
      'administer verify email',
      'administer content types',
      'administer nodes',
    ]);
    $this->drupalLogin($this->adminUser);
    $node_type = NodeType::create([
      'type' => 'page',
      'name' => 'Page',
    ]);
    $node_type->save();
    // Create a node.
    $page = Node::create([
      'title' => 'A beautiful page',
      'type' => 'page',
    ]);
    $page->save();
    // Create a verification form.
    $entity = VerifyEmail::create([
      'label' => 'Verify Email Form',
      'id' => 'verify_email_test',
      'path' => '/verify-email-form',
      'intro' => 'Enter your email address.',
      'email_subject' => 'Your verification email',
      'email_body' => 'Please click this link: [verify-email:url]',
      'thank_you_body' => 'Thank you, you will get an email soon!',
      'redirect_path' => '/node/1',
    ]);
    $entity->save();
    // Create a fake login info object and place in tempstore.
    $tempStore = \Drupal::service('tempstore.shared');
    $store = $tempStore->get('verify_email');
    $store->set(12345, new LoginInfo($entity->id(), 'test@test.com', 'ABCabc01', \Drupal::time()->getCurrentTime() + 3600));
    // Also create an expired login info object.
    $store->set(34343, new LoginInfo($entity->id(), 'expired@test.com', 'MeMeMe34', \Drupal::time()->getCurrentTime() - 3600));
    // Once setup complete, log out.
    $this->drupalLogout();
  }

  /**
   * Test the verify email form is at correct route.
   */
  public function testVerifyEmailRoute(): void {
    $this->drupalGet('/verify-email-form');
    $this->assertSession()->elementExists('xpath', '//h1[text() = "Verify Email Form"]');
    $this->assertSession()->pageTextContains('Enter your email address.');
  }

  /**
   * Test the verify email form for authenticated user.
   */
  public function testVerifyEmailRouteAuthenticatedUser(): void {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/verify-email-form');
    // Test redirection.
    $this->assertSession()->addressEquals('node/1');
    $this->drupalLogout();
  }

  /**
   * Tests invalid email validation.
   */
  public function testInvalidEmail(): void {
    $this->drupalGet('/verify-email-form');
    $this->assertSession()->statusCodeEquals(200);
    // Submit the form with an invalid email address.
    $this->submitForm([
      'email' => 'invalid-email',
    ], 'Verify');
    $this->assertSession()->statusMessageContains('The email address invalid-email is not valid.');
  }

  /**
   * Tests valid email submits form and email content.
   */
  public function testVerifyEmailForm(): void {
    $this->drupalGet('/verify-email-form');
    // Store assertSession in a variable.
    $session = $this->assertSession();

    $session->statusCodeEquals(200);

    // Submit the form with a valid email address.
    $this->submitForm([
      'email' => 'another@test.com',
    ], 'Verify');

    // Check redirection to the 'Thank You' page after form submission.
    $session->statusCodeEquals(200);
    $session->addressEquals('/verify-email-form/thanks');
    $session->elementExists('xpath', '//h1[text() = "Thank you"]');
    $session->pageTextContains('Thank you, you will get an email soon!');

    // Test the email content using AssertMailTrait.
    $this->assertMailString('body', 'Please click this link', 1, 'The email body contains the expected text.');
    $this->assertMailString('body', '/verify/', 1, 'The [verify-email:url] token was replaced with a URL containing /verify/.');
  }

  /**
   * Tests clicking the non existent validation link.
   */
  public function testInvalidLoginLink(): void {
    $this->drupalGet('/verify/54321/xyzXYZ99');
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->addressEquals('');
    $session->statusMessageContains('Sorry, the verification could not be validated. Please try verifying again.');
  }

  /**
   * Tests clicking validation link with incorrect secret.
   */
  public function testInvalidSecretLink(): void {
    $this->drupalGet('/verify/12345/xyzXYZ99');
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->addressEquals('');
    $session->statusMessageContains('Sorry, the verification could not be validated. Please try verifying again.');
  }

  /**
   * Tests clicking expired validation link.
   */
  public function testExpiredLoginLink(): void {
    $this->drupalGet('/verify/34343/MeMeMe34');
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->addressEquals('verify-email-form');
    $session->statusMessageContains('Login link has expired. Please enter your email address again to receive a new link.');
  }

  /**
   * Tests clicking valid validation link.
   */
  public function testValidLoginLink(): void {
    $this->drupalGet('/verify/12345/ABCabc01');
    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->addressEquals('node/1');
    $session->pageTextContains('A beautiful page');
  }

}
